from urllib.parse import urlencode

import models
from api_base import FreekassaApi
from constants import Currencies


class Merchant(FreekassaApi):

    def get_payment_form_url(self, amount: int, order_id: str, currency=Currencies.RUB, payment_method: int = None,
                             phone: str = None, email: str = None, lang: str = None) -> str:
        params = {
            'm': self.shop_id,
            'oa': amount,
            'currency': currency,
            'o': order_id,
            's': self.generate_payment_form_signature(amount, order_id, currency),
            'i': payment_method,
            'phone': phone,
            'em': email,
            'lang': lang,
        }
        payload = {k: v for k, v in params.items() if v is not None}
        return self.INVOICE_BASE_URL.format(query=urlencode(payload))

    def get_balance(self) -> models.Balance:
        resp = self.request('balance')
        currencies_values = {i['currency']: i.get('value') for i in resp['balance']}
        return models.Balance(**currencies_values)

    # def orders(self, order_id: int = None, payment_id: str = None, order_status: int = None,
    #            date_from: str = None, date_to: str = None, page: int = None):
    #     json = {
    #         'orderId': order_id,
    #         'paymentId': payment_id,
    #         'orderStatus': order_status,
    #         'dateFrom': date_from,
    #         'dateTo': date_to,
    #         'page': page,
    #     }
    #     return self.request('orders', json)
    #
    # def create_order(self, i: int, email: str, ip: str, amount: int, currency: str = 'RUB',
    #                  payment_id: str = None, tel: str = None):
    #     json = {
    #         'i': i,
    #         'email': email,
    #         'ip': ip,
    #         'amount': amount,
    #         'currency': currency,
    #         'paymentId': payment_id,
    #         'tel': tel,
    #     }
    #     return self.request('orders/create', json)
    #
    # def currencies(self):
    #     return self.request('currencies')
    #
    # def currency_status(self, payment_method: int):
    #     return self.request(f'currencies/{payment_method}/status')
    #
    # def shops(self):
    #     return self.request('shops')



class PaymentMethods:
    FK_WALLET_RUB = 1
    FK_WALLET_USD = 2
    FK_WALLET_EUR = 3
    VISA_RUB = 4
    YOOMONEY = 6
    VISA_UAH = 7
    MASTERCARD_RUB = 8
    MASTERCARD_UAH = 9
    QIWI = 10
    VISA_EUR = 11
    MIR = 12
    ONLINE_BANK = 13
    USDT_ERC20 = 14
    USDT_TRC20 = 15
    BITCOIN_CASH = 16
    BNB = 17
    DASH = 18
    DOGECOIN = 19
    ZCASH = 20
    MONERO = 21
    WAVES = 22
    RIPPLE = 23
    BITCOIN = 24
    LITECOIN = 25
    ETHEREUM = 26
    STEAMPAY = 27
    MEGAFON = 28
    VISA_USD = 32
    PERFECT_MONEY_USD = 33


class OrderStatuses:
    NEW = 0
    PAID = 1
    ERROR = 8
    CANCELED = 9


class Currencies:
    RUB = 'RUB'
    USD = 'USD'
    EUR = 'EUR'
    UAH = 'UAH'
    KZT = 'KZT'


class Languages:
    RU = 'ru'
    EN = 'en'



#!/usr/bin/env python
from pathlib import Path

from setuptools import find_packages, setup

BASE_DIR = Path(__file__).parent


def get_description():
    with open(BASE_DIR / 'README.md') as f:
        return f.read()


def get_requirements():
    with open(BASE_DIR / 'requirements.txt') as reqs:
        return [r.strip() for r in reqs]


setup(
    name='freekassa_api',
    version='0.0.5',
    url='https://github.com/LDmitriy7/freekassa_api.git',

    packages=find_packages(exclude=('tests', 'tests.*', 'examples.*', 'docs',)),
    install_requires=[
        'certifi==2021.10.8',
        'charset-normalizer==2.0.9',
        'idna==3.3',
        'pydantic==1.8.2',
        'requests==2.26.0',
        'typing-extensions==4.0.1',
        'urllib3==1.26.7',
    ],

    license='MIT',
    author='LDmitriy7',
    author_email='ldm.work2019@gmail.com',

    description='Freekassa API',
    long_description=get_description(),
)



import time

import requests

from helpers import hash_by_sha256, hash_by_md5


class FreekassaApi:
    API_BASE_URL = 'https://api.freekassa.ru/v1/{method}'
    INVOICE_BASE_URL = 'https://pay.freekassa.ru/?{query}'

    def __init__(self, shop_id: int, secret1: str, secret2: str, api_key: str):
        self.session = requests.Session()

        self.shop_id = shop_id
        self.secret1 = secret1
        self.secret2 = secret2
        self.api_key = api_key

    def generate_api_signature(self, json: dict) -> str:
        sorted_values = []

        for key in sorted(json.keys()):
            sorted_values.append(str(json[key]))

        return hash_by_sha256(self.api_key, '|'.join(sorted_values))

    def generate_payment_form_signature(self, amount: int, order_id: str, currency='RUB') -> str:
        return hash_by_md5(f'{self.shop_id}:{amount}:{self.secret1}:{currency}:{order_id}')

    def request(self, method: str, json: dict = None) -> dict:
        url = self.API_BASE_URL.format(method=method)

        json = json or {}
        json = {k: v for k, v in json.items() if v is not None}
        json = {**json, 'shopId': self.shop_id, 'nonce': time.time_ns()}
        json = {**json, 'signature': self.generate_api_signature(json)}

        resp = self.session.post(url, json=json)
        return resp.json()



import hashlib
import hmac


def hash_by_sha256(key: str, string: str) -> str:
    result = hmac.new(key.encode(), string.encode(), digestmod=hashlib.sha256)
    return result.hexdigest()


def hash_by_md5(string: str) -> str:
    return hashlib.md5(string.encode()).hexdigest()
